%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

function clickDownGui(object,~,h,X,y,Theta1,Theta2,widthDigit,heightDigit,rp)

    global idDigit;
    
    %-- If the left button is pressed, then play a forward iteration
    if strcmp(get(object,'selectiontype'),'normal')        
        idDigit = idDigit+1;    
        m = size(X,1);
        if (idDigit>m)
            idDigit=m;
        end
    end
    
    %-- If the right button is pressed, then play a backward iteration    
    if strcmp(get(object,'selectiontype'),'alt')
        idDigit = idDigit-1;    
        if (idDigit<1)
            idDigit=1;
        end                
    end
    
    
    %-- Ramdomly select one image    
    ind = rp(idDigit);
    img = reshape(X(ind,:),[heightDigit widthDigit]);
    
    
    %-- Predict digit thanks to logistic regression classifier
    pred = nn.predict(Theta1, Theta2, X(ind,:));

    
    %-- The digit 0 is identified as label 10
    if (y(ind)==10)
        ref = 0;
    else
        ref = y(ind);
    end
    
    
    %-- Display result
    figure(h); imagesc(img); axis image; colormap(gray); title('Press left/right to navigate'); axis off;
    if (pred == ref)
        text(2,3,{['True value = ',num2str(ref)],['Prediction = ',num2str(pred)]},'Color','green','FontSize',10);
    else
        text(2,3,{['True value = ',num2str(ref)],['Prediction = ',num2str(pred)]},'Color','red','FontSize',10);
    end

end
