%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

function clickUpMakeDigit(~,~)

    %-- Global parameter for the GUI application
    global imgCanevas;
    global digitCanevas;
    
    %-- End up with the callback function related to the 'WindowButtonMotionFcn' event
    set (gcf,'WindowButtonMotionFcn',[]);

    
    %-- Defined a square region centered to the digit 
    %-- and touching the digit at the boundary
    xaxis = sum(imgCanevas,1);
    x = zeros(1,2);
    for i=1:length(xaxis)
        if (xaxis(i)>0)
            x(1,1) = i;
            break;
        end
    end
    for i=length(xaxis):-1:1
        if (xaxis(i)>0)
            x(1,2) = i;
            break;
        end
    end       
    
    yaxis = sum(imgCanevas,2);
    y = zeros(1,2);
    for i=1:length(yaxis)
        if (yaxis(i)>0)
            y(1,1) = i;
            break;
        end
    end
    for i=length(yaxis):-1:1
        if (yaxis(i)>0)
            y(1,2) = i;
            break;
        end
    end          
    
    %-- Crop the image according to the previous defined square region
    dx = x(2)-x(1);
    dy = y(2)-y(1);
    bound = size(imgCanevas,1);    
    if (dx > dy)
        d = dx-dy;
        y0t = y(1) - floor(d/2);
        y1t = y(2) + ceil(d/2);
        if (y0t < 1) 
            y0t = y(1); 
            y1t = y(2) + d;
            if (y1t > bound)
                y1t = bound;
            end
        end
        if (y1t > bound) 
            y0t = y(1) - d; 
            y1t = y(2);
            if (y0t < 1)
                y0t = 1;
            end
        end
        y(1) = y0t;
        y(2) = y1t;
    else        
        d = dy-dx;
        x0t = x(1) - floor(d/2);
        x1t = x(2) + ceil(d/2);
        if (x0t < 1)
            x0t = x(1); 
            x1t = x(2) + d;
            if (x1t > bound)
                x1t = bound;
            end
        end
        if (x1t > bound)
            x0t = x(1) - d;
            x1t = x(2);
            if (x0t < 1)
                x0t = 1;
            end
        end
        x(1) = x0t;
        x(2) = x1t;            
    end
    crop_img =  imgCanevas(y(1):y(2),x(1):x(2));
    
    
    %-- Resize image to [30x30]
    %-- Apply a bilinear interpolation to avoid making appear negative values
    digit = imresize(crop_img,[30 30],'bilinear');
    
    
    %-- Normalize intensity between 0 and 1
    digit = digit - min(digit(:));
    digit = digit / max(digit(:));  
    digitCanevas = digit;
    
    
    %-- Display result on the right subplot
    digitDisplay = zeros(40,40);
    digitDisplay(6:35,6:35) = digit;
    subplot(1,2,2); imagesc(digitDisplay); axis image; colormap(gray); 
    caxis([0 1]); axis off;
    title('Launching digit recognition...');
    
    
    %-- Perform prediction on this new digit
    local_param_path = '../param/';
    if ( exist([local_param_path,'param_mnist_new.mat'],'file') )
        load([local_param_path,'param_mnist_new.mat']);
    else
        load([local_param_path,'param_mnist.mat']);
    end
    x = digit(:)';
    [pred,prob] = nn.predict(Theta1, Theta2, x);
    
    
    %-- Display result    
    title({['Prediction: ',num2str(pred)],['Probability: ',num2str(prob),'%']})
        
end

