%-- This code has been realized to illustrate neural network concepts during 
%-- the workshop organized by the "club EEA" which happened in March 2018 
%-- https://rnn2018lyon.sciencesconf.org/
%--
%-- This code has been strongly inspired by the Mooc of Andrew Ng on coursera
%-- https://www.coursera.org/learn/machine-learning/
%-- 
%-- The application treated here concerns the automatic recognition of 
%-- digits drawn by hand. This code is not protected by any license, you 
%-- can re-use it freely without any restriction. Please do not forget to
%-- cite this event for any use of the code.

clear all;
close all;
clc;
addpath('../');


%-- mnist-like database location
url = 'https://www.creatis.insa-lyon.fr/~bernard/workshop/';
local_data_path = '../data/';
local_param_path = '../param/';


%-- Downlad minst-like database
filename_db = 'mnist.mat';
if (~exist([local_data_path,filename_db],'file'))
     tools.download(filename_db, url, local_data_path);
end


%-- Load mnist-like database
load([local_data_path,filename_db]);
widthDigit = size(training.images,2);
heightDigit = size(training.images,1);


%-- Load learned parameters
filename_param = 'param_mnist.mat';
load([local_param_path,filename_param]);


%-- Create X matrix
X = zeros(size(testing.images,3),widthDigit*heightDigit);
for k=1:size(testing.images,3)
    digit = testing.images(:,:,k);
    X(k,:) = digit(:)';
end
y = testing.labels;
y(y==0) = 10;   %-- the digit 0 is identified as label 10
m = size(X,1);


%-- Evaluate the performance of the learned method from the full testing database
pred = nn.predict(Theta1, Theta2, X);
accuracy = mean(double(pred == y)) * 100;
fprintf('Testing Set Overall Accuracy: %.1f %% \n \n',accuracy);


%-- Evaluate the performance of the learned method for each class digit of the testing database
y(y==10) = 0;
pred(pred==10) = 0;
for k=0:9
    ind = (y==k);
    yD = y(ind);
    predD = pred(ind);
    accuracy = mean(double(predD == yD)) * 100;
    fprintf('\t digit %d [# %d]: %.1f %%\n',k,sum(ind),accuracy);
end

